const usePromise = typeof browser !== "undefined";

/**
 * Global Variables / Config
 */

const mainUrl = "https://access.mymind.com"

let popupCount = 0;
let settingsData = null;

/**
 * Context Menu Init
 */

const menuConfig = [
	{
		title: "Add to my mind",
		id: "awmt-add",
		contexts: ["selection", "image", "video", "audio"]
	},
	{
		title: "Open my mind",
		id: "awmt-open",
		contexts: ["browser_action"]
	},
	{
		title: "Enable Instagram integration",
		id: "awmt-instagram",
		contexts: ["browser_action"]
	},
	/*
	{
		title: "Manage settings",
		id: "awmt-settings",
		contexts: ["browser_action"]
	},
	{
		title: "Disable reminders",
		id: "awmt-toggle",
		contexts: ["browser_action"]
	},
	*/
];

if (usePromise) {
	browser.contextMenus.removeAll().then(() => {
		menuConfig.forEach(config => browser.contextMenus.create(config));
	});
} else {
	chrome.contextMenus.removeAll(() => {
		menuConfig.forEach(config => chrome.contextMenus.create(config));
	});
}

/**
 * Post Install
 */

chrome.runtime.onInstalled.addListener(() => {
	openNewTab(`${mainUrl}/extension`);
});

/**
 * Component Init
 */

chrome.webNavigation.onCompleted.addListener(details => {
	if (details.url.startsWith("https://www.instagram.com/")) {
		function callback(granted) {
			if (granted) initInstagramOverlay(details.tabId);
		}

		checkPermissions({
			origins: ['https://www.instagram.com/*'],
		}, callback);
	}
});

/*
chrome.tabs.onUpdated.addListener(async (tabId, changeInfo, tab) => {
	settingsData = await getSettings();

	const { blockedDomains = [], quickAdd = [] } = settingsData;

	const blocked = [
		"about:blank",
		"chrome://",
		"mymind.com",
		...blockedDomains,
	];

	const isBlocked = blocked.reduce((blocked, item) => blocked || url.includes(item), false);

  if (!isBlocked) {
		if (quickAdd.includes('page')) {
			initReminders(tabId);
		}

		if (quickAdd.includes('highlight')) {
			initPrompts(tabId);
		}

		if (quickAdd.includes('media')) {
			initOverlay(tabId);
		}
	}
});
*/

/**
 * Observers
 */

// Browser Action
chrome.browserAction.onClicked.addListener(getMeta);

// Context Menu
chrome.contextMenus.onClicked.addListener(async (info, tab) => {
	const { menuItemId, selectionText } = info;

	if (menuItemId === "awmt-add") {
		if (selectionText) {
			getSelection();
		} else {
			addToAWMT(info);
		}
		return;
	}

	if (menuItemId === "awmt-open") {
		openNewTab(mainUrl);
		return;
	}

	if (menuItemId === "awmt-settings") {
		openNewTab(`${mainUrl}/extension`);
		return;
	}

	if (menuItemId === "awmt-toggle") {
		const newStatus = await toggleReminders();
		updateToggleStatus(newStatus);
		return;
	}

	if (menuItemId === "awmt-instagram") {
		requestPermissions({
			origins: ['https://www.instagram.com/*']
		});
	}
});

// Runtime Message
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
	const { addTag, openTab, savePage, saveImage, saveText, data = {} } = request;
	const { pageMeta, pageContent, pageUrl } = data;

	if (savePage) {
		if (pageMeta) {
			addToAWMT({ pageUrl: savePage, pageMeta, pageContent });
		} else {
			getMeta();
		}
		return;
	}

	if (saveImage) {
		addToAWMT({ srcUrl: saveImage, pageUrl, pageMeta });
		return;
	}

	if (addTag) {
		addNewTag(request, sendResponse);
		return;
	}

	if (openTab) {
		openNewTab(openTab, sendResponse);
		return;
	}

	if (saveText) {
		addToAWMT({ selectionText: saveText });
		return;
	}
});

/**
 * Request Handlers
 */

 // Parser
async function addToAWMT(info) {
	const { srcUrl, selectionText, pageUrl, pageMeta, pageContent } = info;

	function callback(tab) {
		const source = pageUrl || tab.url;

		if (srcUrl) {
			createObject({ type: "Image", url: srcUrl, source, metadata: pageMeta });
			return;
		}

		if (selectionText) {
			createObject({ content: selectionText, source });
			return;
		}

		if (pageUrl) {
			createObject({ type: "WebPage", url: pageUrl, metadata: pageMeta, html: pageContent });
			return
		}
	}

	if (usePromise) {
		browser.tabs.query({active: true}).then(tab => callback(tab));
	} else {
		chrome.tabs.getSelected(null, tab => callback(tab));
	}
}

// Create New Item
async function createObject(data) {
	try {
		createNotification(
			() =>
				updateNotification({
					message: "Adding to your mind...",
					status: "loading"
				})
		);

		const token = await getAuthToken();

		if (!token) throw 401;

		const response = await fetch(`${mainUrl}/objects`, {
			method: "POST",
			headers: {
				"Content-Type": "application/json",
				"User-Agent": "awmt-chrome/1.0",
				Authorization: "Bearer " + token
			},
			referrer: "no-referrer",
			body: JSON.stringify(data)
		});

		// Created!
		if (response.status === 201) {
			const json = await response.json();

			updateNotification({
				contentId: json.id,
				message: "Added to your mind",
				status: "success",
				redirect: mainUrl
			});
			return;
		}

		if (response.status === 402) {
			updateNotification({
				message: "Your mind is full. Please upgrade your plan.",
				status: "limited",
				redirect: mainUrl
			});
			return;
		}

		if (response.status === 409) {
			updateNotification({
				message: "Whoops! You already added this to your mind.",
				status: "error",
				redirect: mainUrl
			});
			return;
		}

		updateNotification({
			message: "Could not save to mymind. Please try again or check your internet connection",
			status: "error"
		});
	} catch (err) {
		console.log("Oh no, somethings wrong", err);
	}
}

// Add Tag to an Item
async function addNewTag(data, replyToApp) {
	try {
		const token = await getAuthToken();
		const response = await fetch(
			`${mainUrl}/objects/${data.item}/annotations`,
			{
				method: "POST",
				headers: {
					"Content-Type": "application/json",
					"User-Agent": "awmt-chrome/1.0",
					Authorization: "Bearer " + token
				},
				referrer: "no-referrer",
				body: JSON.stringify({ content: data.addTag })
			}
		);

		if (!response.ok) {
			updateNotification({
				message: "Could not save to mymind. Please try again or check your internet connection",
				status: "error"
			});
			return;
		}

		const json = await response.json();

		// Created!
		if (response.status === 201) {
			replyToApp(json);
		}
	} catch (err) {
		console.log("Oh no, somethings wrong", err);
	}
}

// Fetch Settings
async function getSettings() {
	try {
		const token = await getAuthToken();
		const response = await fetch(
			`${mainUrl}/extension/settings.json`,
			{
				method: "GET",
				headers: {
					"Content-Type": "application/json",
					"User-Agent": "awmt-chrome/1.0",
					Authorization: "Bearer " + token
				},
				referrer: "no-referrer"
			}
		);

		if (response.ok) {
			const json = await response.json();
			return json;
		}
	} catch (err) {
		console.log("Oh no, somethings wrong", err);
	}
}

// Toggle Reminders
async function toggleReminders() {
	try {
		const status = settingsData && settingsData.quickAdd && settingsData.quickAdd.length;
		const newStatus = Boolean(!status);
		const token = await getAuthToken();
		const response = await fetch(
			`${mainUrl}/extension/settings`,
			{
				method: "PATCH",
				headers: {
					"Content-Type": "application/json",
					"User-Agent": "awmt-chrome/1.0",
					Authorization: "Bearer " + token
				},
				referrer: "no-referrer",
				body: JSON.stringify({
					quickAddPage: newStatus,
					quickAddHighlight: newStatus,
					quickAddMedia: newStatus
				})
			}
		);

		if (response.ok) {
			return newStatus;
		}

		return true;
	} catch (err) {
		console.log("Oh no, somethings wrong", err);
	}
}

// Get Token
function getAuthToken() {
	function callback(cookie, resolve, fail) {
		if (chrome.runtime.lastError) {
			createNotification(
				() =>
					updateNotification({
						message: "Could not get the login information. Please allow access to mymind website.",
						status: "error"
					})
			);
		}

		if (cookie) {
			resolve(cookie.value);
		} else {
			createNotification(
				() =>
					updateNotification({
						message: "Could not save to mymind. Click here to login.",
						status: "error",
						redirect: `${mainUrl}/signin`
					})
			);
		}
	}

	return new Promise((resolve, fail) => {
		const config = { url: mainUrl, name: "_jwt" };

		if (usePromise) {
			browser.cookies.get(config).then(cookie => callback(cookie, resolve, fail))
		} else {
			chrome.cookies.get(config, cookie => callback(cookie, resolve, fail));
		}
	});
}

/**
 * Utils
 */

function openNewTab(url) {
	if (url) {
		chrome.tabs.create({ url });
	}
}

function updateMenuVisibility(id, status) {
	chrome.contextMenus.update(id, { visible: Boolean(status) })
}

function requestPermissions(permissions, callback) {
	if (usePromise) {
		browser.permissions.request(permissions).then(callback);
	} else {
		chrome.permissions.request(permissions, callback);
	}
}

function checkPermissions(permissions, callback) {
	if (usePromise) {
		browser.permissions.contains(permissions).then(callback);
	} else {
		chrome.permissions.contains(permissions, callback);
	}
}

/**
 * Injected Methods
 */

function getMeta() {
	const config = { file: "utils/get-meta.js" };

	chrome.tabs.executeScript(config);
}

function getSelection() {
	const config = { file: "utils/get-selection.js" };

	chrome.tabs.executeScript(config);
}

/**
 * Components
 */

// Notification Component
function createNotification(callback) {
	chrome.tabs.insertCSS({
		file: "components/notification.css",
	});

	const config = { file: "components/notificationCreate.js" };

	if (usePromise) {
		browser.tabs.executeScript(config).then(callback);
	} else {
		chrome.tabs.executeScript(config, callback);
	}
}

function updateNotification({ status, message, redirect, contentId }) {
	popupCount += 1;

	const props = {
		item: contentId,
		icon: chrome.runtime.getURL("icons/logo.svg"),
		message,
		status,
		redirect,
		id: popupCount
	}

	function callback() {
		chrome.tabs.executeScript({ file: "components/notificationUpdate.js" });
	};

	const config = {
		code: `
			window.awmt = {};
			${Object.entries(props).map(([key, value]) => `window.awmt.${key} = "${value}";`).join('')}
		`
	};

	if (usePromise) {
		browser.tabs.executeScript(config).then(callback);
	} else {
		chrome.tabs.executeScript(config, callback);
	}
}

// Reminder Component
function initReminders(tabId) {
	chrome.tabs.insertCSS(tabId, {
		file: "components/reminder.css",
	});

	const config = {
		file: "components/reminder.js"
	};

	chrome.tabs.executeScript(tabId, config);
}

// Prompt Component
function initPrompts(tabId) {
	chrome.tabs.insertCSS(tabId, {
		file: "components/prompt.css",
	});

	const config = {
		file: "components/prompt.js"
	};

	chrome.tabs.executeScript(tabId, config);
}

// Overlay Component
function initOverlay(tabId) {
	chrome.tabs.insertCSS(tabId, {
		file: "components/overlay.css",
	});

	const config = {
		file: "components/overlay.js"
	};

	chrome.tabs.executeScript(tabId, config);
}

// Instagram Overlay Component
function initInstagramOverlay(tabId) {
	chrome.tabs.insertCSS(tabId, {
		file: "components/overlay.css",
	});

	const config = {
		file: "components/instagramOverlay.js"
	};

	chrome.tabs.executeScript(tabId, config);
}
